from PyQt5.QtCore import PYQT_CONFIGURATION as pyqt_config
from distutils import sysconfig
import os, sipconfig, sys

class HostPythonConfiguration(object):
    def __init__(self):
        self.platform=sys.platform
        self.version=sys.hexversion>>8

        self.inc_dir=sysconfig.get_python_inc()
        self.venv_inc_dir=sysconfig.get_python_inc(prefix=sys.prefix)
        self.module_dir=sysconfig.get_python_lib(plat_specific=1)

        if sys.platform=='win32':
            self.data_dir=sys.prefix
            self.lib_dir=sys.prefix+'\\libs'
        else:
            self.data_dir=sys.prefix+'/share'
            self.lib_dir=sys.prefix+'/lib'

class TargetQtConfiguration(object):
    def __init__(self, qmake):
        pipe=os.popen(' '.join([qmake, '-query']))

        for l in pipe:
            l=l.strip()

            tokens=l.split(':', 1)
            if isinstance(tokens, list):
                if len(tokens) != 2:
                    error("Unexpected output from qmake: '%s'\n" % l)

                name,value=tokens
            else:
                name=tokens
                value=None

            name=name.replace('/', '_')
            setattr(self, name, value)

        pipe.close()        

if __name__=="__main__":
    from argparse import ArgumentParser

    parser=ArgumentParser(description="Configure module.")
    parser.add_argument(
        '-q', '--qmake',
        dest="qmake",
        type=str,
        default="qmake",
        help="Path to qmake executable"
    )
    parser.add_argument(
        '-s', '--sip-extras',
        dest="sip_extras",
        type=str,
        default="",
        help="Extra arguments to sip"
    )
    args=parser.parse_args()

    qmake_exe=args.qmake
    if not qmake_exe.endswith('qmake'):
        qmake_exe=os.path.join(qmake_exe,'qmake')

    if os.system(' '.join([qmake_exe, '-v']))!=0:
        
        if sys.platform=='win32':
            print("Make sure you have a working Qt qmake on your PATH.")
        else:
            print(
                "Use the --qmake argument to explicitly specify a "
                "working Qt qmake."
            )
        exit(1)

    sip_args=args.sip_extras

    pyconfig=HostPythonConfiguration()
    py_sip_dir=os.path.join(pyconfig.data_dir, 'sip', 'PyQt5')
    sip_inc_dir=pyconfig.venv_inc_dir

    qtconfig=TargetQtConfiguration(qmake_exe)

    inc_dir=os.path.abspath(os.path.join(".","src"))
    lib_dir=inc_dir
    dest_pkg_dir="PyOpencascadeDir"
    
    sip_include=os.path.abspath(os.path.join(".","/usr/share/sip/PyQt5/QtGui/"))
    occ_so_include=os.path.abspath(os.path.join(".","/usr/local/lib/"))											# Added skynet
    qopengl_header_include=os.path.abspath(os.path.join(".","/usr/include/x86_64-linux-gnu/qt5/QtOpenGL/"))		# Added skynet
    
    sip_files_dir=os.path.abspath(os.path.join(".","sip"))
    output_dir =os.path.abspath(os.path.join(".", "modules"))
    build_file="PyOpencascade.sbf"
    build_path = os.path.join(output_dir, build_file)
      
    if not os.path.exists(output_dir): os.mkdir(output_dir)
    sip_file = os.path.join(sip_files_dir, "PyOpencascade.sip")

    config=sipconfig.Configuration()    
    config.default_mod_dir=( "/usr/local/lib/python%i.%i/dist-packages" %
                               ( sys.version_info.major, sys.version_info.minor ) )

    cmd=" ".join([
        config.sip_bin,
        pyqt_config['sip_flags'],
        sip_args,
        '-I', sip_include,
        '-I', occ_so_include,			# Added skynet
        '-I', qopengl_header_include,	# Added skynet
        '-I', sip_files_dir,
        '-I', py_sip_dir,
        '-I', config.sip_inc_dir,
        '-I', inc_dir,
        "-c", output_dir,
        "-b", build_path,
        "-w",
        "-o",
        sip_file,
    ])

    print(cmd)
    if os.system(cmd)!=0: sys.exit(1)

    makefile=sipconfig.SIPModuleMakefile(
        config,
        build_file,
        dir=output_dir,
        install_dir=dest_pkg_dir
    )

    makefile.extra_defines+=['Opencascade_LIBRARY','QT_CORE_LIB', 'QT_GUI_LIB', 'QT_WIDGETS_LIB']
    makefile.extra_include_dirs+=[os.path.abspath(inc_dir), qtconfig.QT_INSTALL_HEADERS]
    makefile.extra_lib_dirs+=[qtconfig.QT_INSTALL_LIBS, os.path.join('..','src')]
    makefile.extra_include_dirs+=[os.path.join(".","/usr/local/include/opencascade/")] # Added skynet
    makefile.extra_libs+=['opencascade']

    if sys.platform=='darwin':
        makefile.extra_cxxflags+=['-F'+qtconfig.QT_INSTALL_LIBS]        
        makefile.extra_include_dirs+=[
            os.path.join(qtconfig.QT_INSTALL_LIBS,'QtCore.framework','Headers'),
            os.path.join(qtconfig.QT_INSTALL_LIBS,'QtGui.framework','Headers'),
            os.path.join(qtconfig.QT_INSTALL_LIBS,'QtWidgets.framework','Headers'),
            os.path.join(qtconfig.QT_INSTALL_LIBS,'QtOpenGL.framework','Headers'),
        ]

        makefile.extra_lflags+=[      
            '-F'+qtconfig.QT_INSTALL_LIBS,
            "-framework QtWidgets",
            "-framework QtGui",
            "-framework QtCore",
            "-framework DiskArbitration",
            "-framework IOKit",
            "-framework OpenGL",
            "-framework AGL",
        ]

    else:
        makefile.extra_include_dirs+=[
            os.path.join(qtconfig.QT_INSTALL_HEADERS, "QtCore"),
            os.path.join(qtconfig.QT_INSTALL_HEADERS, "QtGui"),
            os.path.join(qtconfig.QT_INSTALL_HEADERS, "QtWidgets"),
            os.path.join(qtconfig.QT_INSTALL_HEADERS, "QtOpenGL"),
        ]
        makefile.extra_lib_dirs+=[os.path.join('..','src','release')]
        makefile.extra_libs+=['Qt5Core','Qt5Gui','Qt5Widgets','Qt5OpenGL']

    makefile.generate()

    sipconfig.ParentMakefile(
        configuration = config,
        subdirs = ["src", output_dir],
    ).generate()

    os.chdir("src")    
    qmake_cmd=qmake_exe
    if sys.platform=="win32": qmake_cmd+=" -spec win32-msvc2010"
    print()
    print(qmake_cmd)
    os.system(qmake_cmd)
    sys.exit()
